package com.sun.media.sound;

import java.io.File;
import java.io.OutputStream;
import javax.sound.sampled.AudioSystem;
import java.io.IOException;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.spi.AudioFileWriter;

public final class WaveFloatFileWriter extends AudioFileWriter
{
    public WaveFloatFileWriter() {
    }
    
    @Override
    public AudioFileFormat.Type[] getAudioFileTypes() {
        return new AudioFileFormat.Type[] { AudioFileFormat.Type.WAVE };
    }
    
    @Override
    public AudioFileFormat.Type[] getAudioFileTypes(AudioInputStream audioInputStream) {
        if (!audioInputStream.getFormat().getEncoding().equals(AudioFormat.Encoding.PCM_FLOAT)) {
            return new AudioFileFormat.Type[0];
        }
        return new AudioFileFormat.Type[] { AudioFileFormat.Type.WAVE };
    }
    
    private void checkFormat(AudioFileFormat.Type type, AudioInputStream audioInputStream) {
        if (!AudioFileFormat.Type.WAVE.equals(type)) {
            throw new IllegalArgumentException("File type " + type + " not supported.");
        }
        if (!audioInputStream.getFormat().getEncoding().equals(AudioFormat.Encoding.PCM_FLOAT)) {
            throw new IllegalArgumentException("File format " + audioInputStream.getFormat() + " not supported.");
        }
        return;
    }
    
    public void write(AudioInputStream audioInputStream, RIFFWriter riffWriter) throws IOException {
        RIFFWriter writeChunk = riffWriter.writeChunk("fmt ");
        AudioFormat format = audioInputStream.getFormat();
        writeChunk.writeUnsignedShort(3);
        writeChunk.writeUnsignedShort(format.getChannels());
        writeChunk.writeUnsignedInt((long)(int)format.getSampleRate());
        writeChunk.writeUnsignedInt((long)((int)format.getFrameRate() * format.getFrameSize()));
        writeChunk.writeUnsignedShort(format.getFrameSize());
        writeChunk.writeUnsignedShort(format.getSampleSizeInBits());
        writeChunk.close();
        RIFFWriter writeChunk2 = riffWriter.writeChunk("data");
        byte[] b = new byte[1024];
        int read;
        while ((read = audioInputStream.read(b, 0, b.length)) != -1) {
            writeChunk2.write(b, 0, read);
        }
        writeChunk2.close();
    }
    
    private AudioInputStream toLittleEndian(AudioInputStream sourceStream) {
        AudioFormat format = sourceStream.getFormat();
        return AudioSystem.getAudioInputStream(new AudioFormat(format.getEncoding(), format.getSampleRate(), format.getSampleSizeInBits(), format.getChannels(), format.getFrameSize(), format.getFrameRate(), false), sourceStream);
    }
    
    @Override
    public int write(AudioInputStream littleEndian, AudioFileFormat.Type type, OutputStream outputStream) throws IOException {
        this.checkFormat(type, littleEndian);
        if (littleEndian.getFormat().isBigEndian()) {
            littleEndian = this.toLittleEndian(littleEndian);
        }
        RIFFWriter o = new RIFFWriter(new NoCloseOutputStream(outputStream), "WAVE");
        this.write(littleEndian, o);
        int n = (int)o.getFilePointer();
        o.close();
        return n;
    }
    
    @Override
    public int write(AudioInputStream littleEndian, AudioFileFormat.Type type, File file) throws IOException {
        this.checkFormat(type, littleEndian);
        if (littleEndian.getFormat().isBigEndian()) {
            littleEndian = this.toLittleEndian(littleEndian);
        }
        RIFFWriter o = new RIFFWriter(file, "WAVE");
        this.write(littleEndian, o);
        int n = (int)o.getFilePointer();
        o.close();
        return n;
    }
    
    private static class NoCloseOutputStream extends OutputStream
    {
        final OutputStream out;
        
        NoCloseOutputStream(OutputStream out) {
            this.out = out;
        }
        
        @Override
        public void write(int n) throws IOException {
            this.out.write(n);
        }
        
        @Override
        public void flush() throws IOException {
            this.out.flush();
        }
        
        @Override
        public void write(byte[] b, int off, int len) throws IOException {
            this.out.write(b, off, len);
        }
        
        @Override
        public void write(byte[] b) throws IOException {
            this.out.write(b);
        }
    }
}
